local L = LibStub("AceLocale-3.0"):GetLocale("WorldBossStatus")
WorldBossStatus = LibStub("AceAddon-3.0"):NewAddon("WorldBossStatus", "AceConsole-3.0", "AceEvent-3.0" );

local addonName = "WordBossStatus";
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)
local LibQTip = LibStub('LibQTip-1.0')

statuses = {}
statuses[0] = L["Available"]
statuses[1] = L["Defeated"]

colors = {"FF00FF00", "FFFF0000"}
colors[0] = "FF00FF00"
colors[1] = "FFFF0000"

local red = { r = 1.0, g = 0.2, b = 0.2 }
local blue = { r = 0.4, g = 0.4, b = 1.0 }
local green = { r = 0.2, g = 1.0, b = 0.2 }
local yellow = { r = 1.0, g = 1.0, b = 0.2 }
local gray = { r = 0.5, g = 0.5, b = 0.5 }
local black = { r = 0.0, g = 0.0, b = 0.0 }
local white = { r = 1.0, g = 1.0, b = 1.0 }
local frame

local WorldBossStatusLauncher = LDB:NewDataObject(addonName, {
		type = "launcher",
		icon = "Interface\\TARGETINGFRAME\\UI-RaidTargetingIcon_8.png",	
		OnClick = function(clickedframe, button)
			WorldBossStatus:ShowOptions() 
		end,
		OnEnter = function(self)
			frame = self
			RequestRaidInfo()
			WorldBossStatus:ShowToolTip()
		end,
	})
	
local defaults = {
	realm = {
		characters = {
			},
	},
	global = {
		realms = {
			},
		MinimapButton = {
			hide = false,
		}, 
		characterOptions = {
			minimumLevel = 90,
			include = 3,
		},
		bossOptions = {
			hideBoss = {
			},			
		},
	},
};

local options = {
    name = L["World Boss Status"],
    handler = WorldBossStatus,
    type = "group",
    args = {
        showMiniMapButton = {
            type = "toggle",
            name = L["Minimap Button"],
            desc = L["Toggles the display of the minimap button."],
            get = "IsShowMinimapButton",
            set = "ToggleMinimapButton",
			order=1,
        },
		characterOptions = {
			type= "group",
			inline = true,
			name = "Characters",
			desc = "",
			order=2,
			args = {
				includeCharactersOption = {
					type = "select",
					name = L["Include Characters"],
					desc = L["Include the following characters in the display."],
					order = 1,
					values = function()
						options = {
							[1] = L["Active Character Only"],
							[2] = L["On This Realm"],
							[3] = L["On This Account"],
						}
						return options
					end,
					get = function(info)
						return WorldBossStatus.db.global.characterOptions.include
					end,
					set = function(info, value)
						WorldBossStatus.db.global.characterOptions.include = value
					end,
				},
				minimumLevelOption = {
					type = "range",
					name = L["Minimum Level"],
					desc = L["Show characters this level and higher."],
					step = 1, min = 1, max = 90,
					order = 2,
					get = function(info)
						return WorldBossStatus.db.global.characterOptions.minimumLevel
					end,
					set = function(info, value)
						WorldBossStatus.db.global.characterOptions.minimumLevel = value
					end,
					disabled = function()
						return WorldBossStatus.db.global.characterOptions.include == 1
					end,
				},
			},
		},
		bossOptions = {
			type = "group",
			inline = true,
			name = L["World Bosses"],
			desc = "",
			order=3,
			args = {
				showWorldBoss1 = {
					type = "toggle",
					name = WORLD_BOSS_SHA_OF_ANGER,
					desc = L["Toggles the display of this world boss."],
					get = function(info)
						return not WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_SHA_OF_ANGER]
					end,
					set = function(info, value)
						WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_SHA_OF_ANGER] = not value
					end,
					order=1,
				},
				showWorldBoss2 = {
					type = "toggle",
					name = WORLD_BOSS_GALLEON,
					desc = L["Toggles the display of this world boss."],
					get = function(info)
						return not WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_GALLEON]
					end,
					set = function(info, value)
						WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_GALLEON] = not value
					end,
					order=2,
				},
				showWorldBoss3 = {
					type = "toggle",
					name = WORLD_BOSS_NALAK,
					desc = L["Toggles the display of this world boss."],
					get = function(info)
						return not WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_NALAK]
					end,
					set = function(info, value)
						WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_NALAK] = not value
					end,
					order=3,
				},
				showWorldBoss4 = {
					type = "toggle",
					name = WORLD_BOSS_OONDASTA,
					desc = L["Toggles the display of this world boss."],
					get = function(info)
						return not WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_OONDASTA]
					end,
					set = function(info, value)
						WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_OONDASTA] = not value
					end,
					order=4,
				},
				showWorldBoss5 = {
					type = "toggle",
					name = WORLD_BOSS_FOUR_CELESTIALS,
					desc = L["Toggles the display of this world boss."],
					get = function(info)
						return not WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_FOUR_CELESTIALS]
					end,
					set = function(info, value)
						WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_FOUR_CELESTIALS] = not value
					end,
					order=5,
				},
				showWorldBoss6 = {
					type = "toggle",
					name = WORLD_BOSS_ORDOS,
					desc = L["Toggles the display of this world boss."],
					get = function(info)
						return not WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_ORDOS]
					end,
					set = function(info, value)
						WorldBossStatus.db.global.bossOptions.hideBoss[WORLD_BOSS_ORDOS] = not value
					end,
					order=6,
				},
			},
		},
    },
}
function WorldBossStatus:GetOtherCharacters(realm)
	characters = {}

	return characters
end 

function WorldBossStatus:DisplayCharacterInTooltip(characterName, characterInfo)
	local bosses = { WORLD_BOSS_SHA_OF_ANGER, WORLD_BOSS_GALLEON, WORLD_BOSS_NALAK, WORLD_BOSS_OONDASTA, WORLD_BOSS_FOUR_CELESTIALS, WORLD_BOSS_ORDOS}
	local tooltip = WorldBossStatus.tooltip

	local line = tooltip:AddLine()
	local factionIcon = "|TInterface\\Icons\\Inv_Misc_QuestionMark:14|t"

	if characterInfo.faction and characterInfo.faction == "Alliance" then
		factionIcon = "|TInterface\\Icons\\Inv_Misc_Tournaments_banner_Human:14|t"
	elseif characterInfo.faction and characterInfo.faction == "Horde" then
		factionIcon = "|TInterface\\Icons\\Inv_Misc_Tournaments_banner_Orc:14|t"
	end

	tooltip:SetCell(line, 2, factionIcon.." "..characterName)

	column = 3
	for i = 1, 6  do
		if not WorldBossStatus.db.global.bossOptions.hideBoss[bosses[i]] then		
			local boss = bosses[i]
			local defeated = (characterInfo.bossKills[boss] and characterInfo.bossKills[boss] > time())

			tooltip:SetCell(line, column, statuses[defeated and 1 or 0])

			if defeated then 
				tooltip:SetCellTextColor(line, column, red.r, red.g, red.b)
			else
				tooltip:SetCellTextColor(line, column, green.r, green.g, green.b)
			end

			if characterInfo.class then
				local color = RAID_CLASS_COLORS[characterInfo.class]
				tooltip:SetCellTextColor(line, 2, color.r, color.g, color.b)
			end

			column = column+1
		end
	end
end


function WorldBossStatus:IsShowMinimapButton(info)
	return not self.db.global.MinimapButton.hide
end

function WorldBossStatus:ToggleMinimapButton(info, value)
	self.db.global.MinimapButton.hide = not value
end

function WorldBossStatus:IsShowObjectives(info)
	return not self.db.global.Objectives.hide
end

function WorldBossStatus:ToggleObjectives(info, value)
	self.db.global.Objectives = not value	
end

function WorldBossStatus:ShowOptions()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function WorldBossStatus:OnInitialize()	
	self.db = LibStub("AceDB-3.0"):New("WorldBossStatusDB", defaults, true)

	LDBIcon:Register(addonName, WorldBossStatusLauncher, self.db.global.MinimapButton)
	WorldBossStatus:RegisterChatCommand("wbs", "ChatCommandHandler")
	LibStub("AceConfig-3.0"):RegisterOptionsTable(L["World Boss Status"], options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(L["World Boss Status"], L["World Boss Status"])
	RequestRaidInfo()
end

function WorldBossStatus:ChatCommandHandler(input)
  if not input or input:trim() == "" then
	  	--frame = self
		RequestRaidInfo()
		--WorldBossStatus:ShowToolTip()
		WorldBossStatus:ShowStatusInChatWindow()  
    else 
        WorldBossStatus:ShowOptions()
    end
end

function WorldBossStatus:ShowStatusInChatWindow()
	local bosses = { { name = WORLD_BOSS_SHA_OF_ANGER, ID = 1},
			   { name = WORLD_BOSS_GALLEON, ID = 2},
			   { name = WORLD_BOSS_NALAK, ID = 3},
			   { name = WORLD_BOSS_OONDASTA, ID = 4},
			   { name = WORLD_BOSS_FOUR_CELESTIALS, ID = 5},
			   { name = WORLD_BOSS_ORDOS, ID = 6}}

	local statuses = {};

	for x = 1, 6 do
		statuses[x] = L["Available"]
	end


	local  num = GetNumSavedWorldBosses()
	for i = 1, num do
		name, worldBossID, reset = GetSavedWorldBossInfo(i)
		statuses[worldBossID] = L["Defeated"]

	end

	for x = 1, 6 do
		self:Print(bosses[x].name.." "..statuses[x])
	end
end

function WorldBossStatus:DisplayRealmInToolip(realmName)
	local realmInfo = self.db.global.realms[realmName]
	local characters = nil
	local collapsed = false

	if realmInfo then
		characters = realmInfo.characters
		collapsed = realmInfo.collapsed
	end

	
	local characterNames = {}
	local currentCharacterName = UnitName("player")
	local currentRealmName = GetRealmName()
	local tooltip = WorldBossStatus.tooltip
	local minimumLevel = WorldBossStatus.db.global.characterOptions.minimumLevel
	local bosses = { WORLD_BOSS_SHA_OF_ANGER, WORLD_BOSS_GALLEON, WORLD_BOSS_NALAK, WORLD_BOSS_OONDASTA, WORLD_BOSS_FOUR_CELESTIALS, WORLD_BOSS_ORDOS}

	if not minimumLevel then minimumLevel = 90 end
	
	if not characters then
		return 
	end

	for k,v in pairs(characters) do
		if (realmName ~= currentRealmName or k ~= currentCharacterName) and (v.level >= minimumLevel) then
			table.insert(characterNames, k);
		end
	end

	if (table.getn(characterNames) == 0) then
		return
	end
			   
	table.sort(characterNames)

	tooltip:AddSeparator(2,0,0,0,0)

	if not collapsed then
		line = tooltip:AddHeader("|TInterface\\Buttons\\UI-MinusButton-Up:16|t", realmName)

		column = 3
		for i = 1, 6  do
			if not WorldBossStatus.db.global.bossOptions.hideBoss[bosses[i]] then
				tooltip:SetCell(line, column, bosses[i])
				tooltip:SetCellTextColor(line, column, yellow.r, yellow.g, yellow.b)

				column = column +1
			end
		end
		

		tooltip:AddSeparator(3,0,0,0,0)

		for k,v in pairs(characterNames) do
			WorldBossStatus:DisplayCharacterInTooltip(v, characters[v])
		end
	
		tooltip:AddSeparator(1, 1, 1, 1, 1.0)
	else
		line = tooltip:AddHeader("|TInterface\\Buttons\\UI-PlusButton-Up:16|t", realmName)
	end

	tooltip:SetCellTextColor(line, 2, yellow.r, yellow.g, yellow.b)	
	tooltip:SetCellScript(line, 1, "OnMouseUp", RealmOnClick, realmName)
end

function RealmOnClick(cell, realmName)
	WorldBossStatus.db.global.realms[realmName].collapsed = not WorldBossStatus.db.global.realms[realmName].collapsed
	WorldBossStatus:ShowToolTip()
end

function WorldBossStatus:ShowToolTip()

	local tooltip = WorldBossStatus.tooltip
	local characterName = UnitName("player")
	local bossKills = WorldBossStatus:GetWorldBossKills()
	local characters = WorldBossStatus.db.realm.characters
	local bosses = { WORLD_BOSS_SHA_OF_ANGER, WORLD_BOSS_GALLEON, WORLD_BOSS_NALAK, WORLD_BOSS_OONDASTA, WORLD_BOSS_FOUR_CELESTIALS, WORLD_BOSS_ORDOS}
	local class, className = UnitClass("player")
	local includeCharacters = WorldBossStatus.db.global.characterOptions.include or 3

	if LibQTip:IsAcquired("WorldBossStatusTooltip") and tooltip then
		tooltip:Clear()
	else
		bossCount = 0
		for i = 1, 6  do				
			if not WorldBossStatus.db.global.bossOptions.hideBoss[bosses[i]] then
				bossCount = bossCount + 1
			end
		end

		tooltip = LibQTip:Acquire("WorldBossStatusTooltip", 2 + bossCount, "CENTER", "LEFT", "CENTER", "CENTER", "CENTER", "CENTER", "CENTER", "CENTER")
		WorldBossStatus.tooltip = tooltip 
	end

	


	line = tooltip:AddHeader(" ")
	tooltip:SetCell(1, 1, "|TInterface\\TARGETINGFRAME\\UI-RaidTargetingIcon_8:16|t "..L["World Boss Status"], nil, "LEFT", tooltip:GetColumnCount())
			
	tooltip:AddSeparator(6,0,0,0,0)
   
	line = tooltip:AddHeader()
	tooltip:SetCell(line, 2, L["Character"], nil, nil, nil, nil, nil, 50)
	tooltip:SetCellTextColor(line, 2, yellow.r, yellow.g, yellow.b)


	column = 3
	for i = 1, 6  do				
		if not WorldBossStatus.db.global.bossOptions.hideBoss[bosses[i]] then
			tooltip:SetCell(line, column, bosses[i], "CENTER")
			tooltip:SetCellTextColor(line, column, yellow.r, yellow.g, yellow.b)
			column = column+1
		end
	end

	tooltip:AddSeparator(6,0,0,0,0)
			
	local info = WorldBossStatus:GetCharacterInfo()
	WorldBossStatus:DisplayCharacterInTooltip(characterName, info)

	tooltip:AddSeparator(6,0,0,0,0)
	tooltip:AddSeparator(1, 1, 1, 1, 1.0)

	if includeCharacters > 1 then
			
		WorldBossStatus:DisplayRealmInToolip(GetRealmName())

	end
			
			
			
	if includeCharacters == 3 then
		realmNames = {}
				
		for k,v in pairs(WorldBossStatus.db.global.realms) do
			if (k ~= GetRealmName()) then
				table.insert(realmNames, k);
			end
		end
				
		for k,v in pairs(realmNames) do
			WorldBossStatus:DisplayRealmInToolip(v)
		end
	end


	line = tooltip:AddLine(" ")

	tooltip:SetCell(tooltip:GetLineCount(), 1, L["Click to open the options menu"], nil, "LEFT", tooltip:GetColumnCount())

	if (frame) then
		tooltip:SetAutoHideDelay(0.01, frame)
		tooltip:SmartAnchorTo(frame)
	end 

	tooltip:UpdateScrolling()
	tooltip:Show()
end

function WorldBossStatus:SaveCharacterInfo()
	local characterName = UnitName("player")
	local realmName = GetRealmName()
	
	if not self.db.global.realms then
		self.db.global.realms = {}
	end

	local realmInfo = self.db.global.realms[realmName]

	if not realmInfo then
		realmInfo = {}
		realmInfo.characters = {}
	end
	
	realmInfo.characters[characterName]  = WorldBossStatus:GetCharacterInfo()


	self.db.global.realms[realmName] = realmInfo
end

function WorldBossStatus:GetCharacterInfo()
	local characterInfo = {}
	local class, className = UnitClass("player")
	local level = UnitLevel("player")
	local englishFaction, localizedFaction = UnitFactionGroup("player")

	characterInfo.bossKills = WorldBossStatus:GetWorldBossKills()
	characterInfo.lastUpdate = time()
	characterInfo.class = className
	characterInfo.level = level
	characterInfo.faction = englishFaction

	return characterInfo
end

function WorldBossStatus:GetWorldBossKills()
	local bossKills = {}
	local num = GetNumSavedWorldBosses()

	for i = 1, num do
		name, worldBossID, reset = GetSavedWorldBossInfo(i)
		expires = time() + reset

		bossKills[name] = expires
	end

	return bossKills
end

function WorldBossStatus:UPDATE_INSTANCE_INFO()
	WorldBossStatus:SaveCharacterInfo()
	if LibQTip:IsAcquired("WorldBossStatusTooltip") and WorldBossStatus.tooltip then
		WorldBossStatus:ShowToolTip()
	end
end

oldLogout = Logout;
oldQuit = Quit;
function Quit()
	RequestRaidInfo();
	oldQuit();
end
function Logout()
	RequestRaidInfo();
	oldLogout();
end

function WorldBossStatus:OnEnable()
	self:RegisterEvent("UPDATE_INSTANCE_INFO");
end

function WorldBossStatus:OnDisable()
    -- Called when the addon is disabled
end
